// file: $USFS/class/image_analysis/v1.0/image_analysis.h
//

// make sure the definitions are made only once
//
#ifndef __ISIP_IMAGE_ANALYSIS
#define __ISIP_IMAGE_ANALYSIS

// isip include files
//
#ifndef __ISIP_INTEGRAL
#include <integral.h>
#endif

// forward class definitions
//
class Ppm;

// Image_analysis: a class that measures the scenic beauty of a ppm image
//
class Image_analysis {

  //---------------------------------------------------------------------------
  //
  // protected members
  //
  //---------------------------------------------------------------------------
protected:

  // declare a variable for debugging
  //
  int_4 debug_level_d;
  
  // filename information
  //
  char_1* param_version_d;
  char_1* param_fname_d;   
  char_1* model_version_d;
  char_1* model_fname_d;
  
  // parameters related to the choice of analysis algorithm and mode
  //
  int_4 algorithm_d;
  int_4 mode_d;
  
  //===========================================================================
  //
  // parameters related to histogram-based processing
  // 
  //===========================================================================
  //
  // declare the reference model parameters
  //
  float_8 hist_min_d;        // reference minimum value from the model file
  float_8 hist_max_d;        // reference maximum value from the model file
  int_4 hist_nbins_d;        // reference number of bins from the model file
  int_4 hist_redp_d;         // index of the red color from the model file
  int_4 hist_grnp_d;         // index of the green color from the model file
  int_4 hist_blup_d;         // index of the blue color from the model file

  int_4 hist_r_npxls_d;      // reference number of pixels
  int_4 hist_r_nimages_d;    // number of images processed in this model
  float_8* hist_r_rpdf_d;    // reference distribution of red 
  float_8* hist_r_gpdf_d;    // reference distribution of green
  float_8* hist_r_bpdf_d;    // reference distribution of blue
  float_8* hist_r_rcval_d;   // center values of the colors
  float_8* hist_r_gcval_d;
  float_8* hist_r_bcval_d;
  
  // declare parameters involved in scaling distance measures
  //
  float_8 hist_rwgt_d;       // weights to relate to the SBE
  float_8 hist_gwgt_d;
  float_8 hist_bwgt_d;
  
  float_8 hist_ascale_d;       // scaling functions
  float_8 hist_bscale_d;

  // declare space for the results of computing features on new images
  //
  int_4 hist_t_npxls_d;      // number of pixels tested
  int_4 hist_t_nimages_d;    // number of images tested
  float_8* hist_t_rpdf_d;    // computed distribution of red
  float_8* hist_t_gpdf_d;    // computed distribution of green
  float_8* hist_t_bpdf_d;    // computed distribution of blue
  float_8* hist_t_rcval_d;   // computed center values of the colors
  float_8* hist_t_gcval_d;
  float_8* hist_t_bcval_d;
  
  //===========================================================================
  //
  // parameters related to edge detection processing
  // 
  //===========================================================================

  // declare reference parameters
  //
  int_4 edge_nbins_d;       // reference number of bins from the model file 
  float_8 edge_xmean_d;     // reference xmean from the model file
  float_8 edge_ymean_d;     // reference ymean from the model file
  float_8 edge_sigma_d;     // reference sigma from the model file
  int_4 edge_msize_d;       // reference mask size from the model file
  float_8 edge_rfact_d;     // reference ratio factor from the model file
  float_8 edge_lmin_d;      // minimum value of the length of the line
  float_8 edge_lmax_d;      // maximum value of the length of the line
  
  float_8* edge_r_lpdf_d;   // reference distribution of the lines
  float_8* edge_r_lcval_d;  // reference center values of the bin
  int_4 edge_r_tlines_d;    // reference total number of lines
  int_4 edge_r_nimages_d;   // reference total number of images processed

  // declare parametets involved in scaling measures
  //
  float_8 edge_ascale_d;    // scaling functions and weights that relate to SBE
  float_8 edge_bscale_d;
  float_8 edge_lwgt_d;
  float_8 edge_swgt_d;
  
  // declare space for the results of computing features on the images
  //
  float_8* edge_t_lpdf_d;           // computed distribution of lines
  float_8* edge_t_lcval_d;          // computed center values of the bins
  int_4 edge_t_tlines_d;            // computed number of lines
  int_4 edge_t_nimages_d;           // number of images
  float_8 edge_perlong_lines_d;
  float_8 edge_pershrt_lines_d;
  
  //---------------------------------------------------------------------------
  //
  // public members
  //
  //---------------------------------------------------------------------------
public:
  
  // required methods
  //
  char_1* name_cc();
  volatile void error_handler_cc(char_1* method_name, char_1* message);
  logical_1 debug_cc(FILE* fp, char_1* message);
  logical_1 set_debug_cc(int_4 debug_level);
  
  // destructor/constructor
  //
  ~Image_analysis();
  Image_analysis();
  
  // load/write  methods
  //
  logical_1 load_cc(char_1* param_file);
  logical_1 load_model_cc();
  logical_1 write_cc();
  
  // initialize methods
  //
  logical_1 init_cc();
  
  // methods for processing multiple model files
  //
  logical_1 get_nmods_cc(int_4& num_models, char_1**& param_file,
			 char_1* list_param_file);
  
  // process methods: these methods control the type of computation
  //
  logical_1 process_cc(float_8& sbe_value, char_1* image_fname);
  logical_1 test_cc(float_8& sbe_value, char_1* image_fname);
  logical_1 train_cc(char_1* image_fname);
  logical_1 update_cc(float_8& sbe_value, char_1* image_fname);
  
  // display methods
  //
  logical_1 display_model_cc(FILE* fp);
  
  //---------------------------------------------------------------------------
  //
  // private members
  //
  //---------------------------------------------------------------------------
private:
  
  // input/output methods
  //
  FILE* open_cc(char_1* filename, int_4 mode_a);
  logical_1 close_cc(FILE* fp);
  logical_1 hist_write_cc();
  logical_1 edge_model_write_cc();
  logical_1 edge_write_cc(float_4* out_image, char_1* fname_a,
			  int_4 width, int_4 height);
  
  // get methods
  //
  logical_1 get_mode_cc(FILE* fp_p);
  logical_1 get_algorithm_cc(FILE* fp_p);
  logical_1 get_modelfile_cc(FILE* fp_p);
  
  //===========================================================================
  //
  // histogram-related methods
  //
  //===========================================================================
  //
  // load methods
  //
  logical_1 hist_load_cc();

  // computation methods
  //
  logical_1 hist_test_cc(float_8& sbe_value, char_1* image_fname);
  logical_1 hist_train_cc(char_1* image_fname);
  logical_1 hist_compute_cc(char_1* image_fname);
  logical_1 hist_compare_cc(float_8& sbe_value);

  // lower-level methods
  //
  logical_1 hist_assign_cc(uint_1* b_red, uint_1* b_green, uint_1* b_blue,
			   int_4 npixels);
  int_4 hist_compute_index_cc(float_8 value,
			      float_8 min, float_8 max, float_8 scale);
  logical_1 hist_display_cc(FILE* fp);
  logical_1 hist_init_cc();
  logical_1 hist_normalize_cc();
  
  //===========================================================================
  //
  // edge detection-related methods
  //
  //===========================================================================
  //
  // load methods
  //
  logical_1 edge_load_cc();
  
  // computation methods
  //
  logical_1 edge_test_cc(float_8& sbe_value, char_1* image_fname);
  logical_1 edge_train_cc(char_1* image_fname_a);
  logical_1 edge_compute_cc(char_1* image_fname);
  logical_1 edge_compare_cc(float_8& sbe_value);

  // lower level methods
  //
  logical_1 edge_init_cc();
  logical_1 edge_gauss_cc(float_4* edge_gauss);
  logical_1 edge_derivatives_cc(float_4* edge_xderiv, float_4* edge_yderiv);
  logical_1 edge_gradient_cc(float_4* grad_output, int_4 width, int_4 height,
			     float_4* xderiv_image, float_4* yderiv_image);
  logical_1 edge_convolve_cc(float_4* convol_output, float_4* convol_buf1,
			     float_4* convol_buf2, int_4 width, int_4 height);
  logical_1 edge_maxsup_cc(float_4* out_image, float_4* grad_output,
			   int_4 width, int_4 height);
  logical_1 edge_lines_cc(int_4* num_lines, float_4* edgedet_data_a,
			  int_4 width_a, int_4 height_a);
  logical_1 edge_gray_cc(float_4* gray_output, int_4 width, int_4 height,
			 uint_1* b_red, uint_1* b_green, uint_1* b_blue);
  logical_1 edge_assign_cc(int_4 num_lines);
  logical_1 edge_normalize_cc();
    
};

// end of file
//
#endif
		    
